@@ -62,28 +62,35 @@ proc register_undoable_cmd {t code {name ""} {marks ""}} {
 
 # Selection code
 
+set kill_mark ""
+
 proc kill_next_line {t} {
 	global modified;	set modified 1
+	global kill_mark
 	if {([$t get insert] == "\n")} {
 		set end {insert +1 chars}
 	} else {set end {insert lineend}}
 	set killed [$t get insert $end]
 	$t delete insert $end
+	set offset [string length $killed]
+	catch {.t_kill tag remove sel sel.first sel.last}
 
-	if {[$t compare insert != kill_mark]} {
-		.t_kill delete 1.0 end
-		$t mark set kill_mark insert
+	if {($kill_mark == "") || [$t compare insert != $kill_mark]
+		|| ![catch {.t_kill indes sel.first}]} {
+		catch ".t_kill delete killed.first killed.last"
+		set kill_mark [$t index insert]
 	}
 
-	catch {.t_kill tag remove sel sel.first sel.last}
-	.t_kill insert end $killed
-	.t_kill tag add sel 1.0 end
+	if {[catch ".t_kill index killed.last" index]} {set index 1.0}
+	.t_kill insert $index $killed
+	.t_kill tag add sel 1.0 "$index +$offset chars"
+	.t_kill tag add killed 1.0 "$index +$offset chars"
 }
 
 proc kill_region {t} {
 	global modified	;	set modified 1
 	if {![catch {set m [$t index mark]}]} {
-		.t_kill delete 1.0 end
+		catch {.t_kill delete killed.first killed.last}
 		if {[$t compare $m <= insert]} {
 			set start $m
 			set end insert
@@ -91,7 +98,9 @@ proc kill_region {t} {
 			set end $m}
 		catch {$t tag remove sel sel.first sel.last}
 		.t_kill insert 1.0 [$t get $start $end]
-		.t_kill tag add sel 1.0 end
+		set offset [string length [$t get $start $end]]
+		.t_kill tag add sel 1.0 "1.0 +$offset chars"
+		.t_kill tag add killed 1.0 "1.0 +$offset chars"
 		$t delete $start $end
 }}
 
@@ -233,7 +242,7 @@ proc self_insert {t {c ""}} {
 }
 
 proc delete_range {t start end {dont_undo ""}} {
-	if {([$t compare $start == $end])} {beep ; return}
+	if {([$t compare $start >= $end])} {beep ; return}
 
 	if {($dont_undo == "")} {
 		# Figure out how to undo delete
@@ -318,7 +327,7 @@ proc editbind {f t m} {
 space			{} \
 Key			{self_insert %W %A} \
 $Keys(C_Delete)		{delete_selection %W} \
-C-c			{+catch {.t_kill delete 1.0 end}} \
+C-c			{+catch {.t_kill delete killed.first killed.last}} \
 M-c			{filter_word %W string_capitalize} \
 C-d			{delete_range %W insert {insert +1 char} 1} \
 M-d			{delete_range %W insert {insert +1 chars wordend}} \
@@ -371,3 +380,4 @@ pack append $frame $frame.fmb {left}
 $text mark set kill_mark 1.0
 catch {text .t_kill}
 .t_kill delete 1.0 end
+.t_kill insert 1.0 "\n"
