@@ -1,5 +1,70 @@
 # Bindings for incremental and regexp search
 
+# Functionally equivalent to saying [string first $string [$t get $start $end]]
+# except the index of the found string is returned, not just a character offset.
+# However, it runs faster when $start and $end are far apart and $string is
+# close to $start.
+#
+# It seems that [$t get $start $end] is slow when start and end are far apart.
+# So this routine searches first the line after start,
+# then the next line after it, then the next two lines, the next four lines,
+# on to the end, returning whenever the string is found.
+# Since it does a line-by-line search, the string cannot contain newlines.
+proc text_string_first {t string {start 1.0} {end end}} {
+	set start [$t index $start]
+	set end [$t index $end]
+	if {[$t compare "$start lineend" == "$end lineend"]} {
+		set first_end $end
+	} else {set first_end "$start lineend"}
+	set result [string first $string [$t get $start $first_end]]
+	if {$result != -1} {return "$start +$result chars"}
+# Not on line with start, start traversing down.
+	scan $start "%d.%d" row dummy
+	scan $end "%d.%d" end_row dummy
+	if {$row == $end_row} {return ""}
+	set incr_factor 1
+	for {incr row} {[expr $row + $incr_factor] < $end_row} \
+			{incr row $incr_factor
+			set incr_factor [expr $incr_factor * 2]} {
+		set result [string first $string [$t get "$row.0" \
+				"$row.0 +$incr_factor lines -1c"]]
+		if {$result != -1} {return "$row.0 + $result chars"}
+	}
+# Not on any line, except maybe last.
+	set result [string first $string [$t get "$row.0" $end]]
+	if {$result != -1} {return "$row.0 + $result chars"
+	} else {return ""}
+}
+
+# Like [string last [$t get $start $end]], but more efficient if $string is
+# close to $end.
+proc text_string_last {t string {end end} {start 1.0}} {
+	set start [$t index $start]
+	set end [$t index $end]
+	if {[$t compare "$start linestart" == "$end linestart"]} {
+		set first_start $start
+	} else {set first_start "$end linestart"}
+	set result [string last $string [$t get $first_start $end]]
+	if {$result != -1} {return "$first_start +$result chars"}
+# Not on line with end, start traversing up
+	scan $start "%d.%d" start_row dummy
+	scan $end "%d.%d" row dummy
+	if {$row == $start_row} {return ""}
+	set incr_factor 1
+	for {incr row -1} {[expr $row - $incr_factor] > $start_row} \
+			{set incr_factor [expr $incr_factor * 2]
+			incr row -$incr_factor} {
+		set result [string last $string [$t get "$row.0" \
+				"$row.0 +$incr_factor lines -1c"]]
+		if {$result != -1} {return "$row.0 + $result chars"}
+	}
+# Not on any line, except maybe first
+	incr row $incr_factor
+	set result [string last $string [$t get $start "$row.0 -1c"]]
+	if {$result != -1} {return "$start + $result chars"
+	} else {return ""}
+}
+
 
 # Find_forward and find_reverse can be used by other procedures; they have
 # no visible effects.
@@ -7,17 +72,17 @@
 # Finds first occurrence of string after point in text widget t
 # Returns start and end of occurance or "" if unsuccessful.
 proc find_forward {t point string} {
-	set answer [string first $string [$t get $point end]]
-	if {($answer < 0)} {return ""
-	} else {return [list "$point +$answer chars" "$point +$answer chars +[string length $string] chars"]
+	set answer [text_string_first $t $string $point]
+	if {($answer == "")} {return ""
+	} else {return [list "$answer" "$answer +[string length $string] chars"]
 }}
 
 # Finds first occurrence of string before point in text widget t
 # Returns start and end of occurance or "" if unsuccessful.
 proc find_reverse {t point string} {
-	set answer [string last $string [$t get 1.0 $point]]
-	if {($answer < 0)} {return ""
-	} else {return [list "1.0 +$answer chars" "1.0 +$answer chars +[string length $string] chars"]
+	set answer [text_string_last $t $string $point]
+	if {($answer == "")} {return ""
+	} else {return [list $answer "$answer +[string length $string] chars"]
 }}
 
 # A regular expression searcher
